'use strict';

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var _require = require('./constants'),
    EXECUTION_THRESHOLD = _require.EXECUTION_THRESHOLD;

var _require2 = require('./utils'),
    getInitData = _require2.getInitData,
    getImportDeclaration = _require2.getImportDeclaration;

function BabelPluginFunctionInjector(babel) {
	var t = babel.types;


	var exptNamedExeCount = 0;

	return {
		visitor: {
			Program: function Program(path) {
				var _getInitData = getInitData(this),
				    params = _getInitData.params;

				var regex = new RegExp(params.target);
				if (!regex.test(this.file.opts.filename)) return;

				var _iteratorNormalCompletion = true;
				var _didIteratorError = false;
				var _iteratorError = undefined;

				try {
					for (var _iterator = params.imports[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
						var imp = _step.value;

						var imprtDecl = getImportDeclaration(imp, t);
						path.unshiftContainer('body', imprtDecl);
					}
				} catch (err) {
					_didIteratorError = true;
					_iteratorError = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}
					} finally {
						if (_didIteratorError) {
							throw _iteratorError;
						}
					}
				}
			},
			ExportNamedDeclaration: function ExportNamedDeclaration(path) {
				var _getInitData2 = getInitData(this),
				    params = _getInitData2.params,
				    func = _getInitData2.func;

				if (exptNamedExeCount >= EXECUTION_THRESHOLD) return;
				exptNamedExeCount++;

				var importDeclaration = getImportDeclaration({
					name: 'Component',
					lib: path.node.source.value,
					defaultImport: true
				}, t);

				var _program = path.findParent(function (p) {
					return p.isProgram();
				});
				_program.unshiftContainer('body', importDeclaration);

				var arr = [].concat(_toConsumableArray(params.imports));
				arr.shift();

				var callExp = arr.reduce(function (acc, curr) {
					return t.callExpression(acc, [t.identifier(curr.name)]);
				}, t.identifier(func.name));

				var componentIdentifier = t.identifier('Component');

				callExp = t.callExpression(callExp, [componentIdentifier]);

				var componentWrapperDeclarator = t.variableDeclarator(t.identifier('componentWrapper'), callExp);

				var componentWrapper = t.variableDeclaration('const', [componentWrapperDeclarator]);

				path.node.declaration = componentWrapper;
			},
			ExportDefaultDeclaration: function ExportDefaultDeclaration(path) {
				var _getInitData3 = getInitData(this),
				    params = _getInitData3.params,
				    func = _getInitData3.func;

				var arr = [].concat(_toConsumableArray(params.imports));
				arr.shift();

				var callExp = arr.reduce(function (acc, curr) {
					return t.callExpression(acc, [t.identifier(curr.name)]);
				}, t.identifier(func.name));
				callExp = t.callExpression(callExp, [path.node.declaration]);
				path.node.declaration = callExp;
			}
		}
	};
}

module.exports = BabelPluginFunctionInjector;